#!/bin/tclsh
#
# v_pre.tcl, SITE PRE for glftpd
set revision "Dec/20/2000"

#
# official homepage      : http://vware.hypermart.net
# author's email         : moopowah@yahoo.com
#
#
# :: Description ::
#     this pre script updates the userfile and gives credits only to the
#     user who issues the command (the code to update stats is commented
#     out because i don't feel prewhores deserve all those stats ;).
#     it doesn't care who originally uploaded the files (i.e., file
#     ownership).  groups can have multiple privpaths and pre to multiple
#     sections.  search for "log" in this file and you can edit the
#     logging output to glftpd.log as you wish.  some like to format the
#     output in the pre script and just have their irc bot output it
#     without changes (i'd personally call the trigger RAW:, but since
#     i've seen this treated as PRE:, i use the same here).  this script
#     is mainly for someone who knows tcl and wants to hack the code.
#
# :: Requirements ::
#     1) unix utils: time (this is usually found at /usr/bin/time).  a
#        good choice is: cp /usr/bin/time /glftpd/bin/.
#     2) your logfile should be writable from this program, so either set
#        this with a suid bit (not preferred), or make the file writable:
#        chmod 666 /glftpd/ftp-data/logs/glftpd.log
#
# :: Installation ::
#      1) cp vPre.tcl /glftpd/bin/
#      2) add this to glftpd.conf: site_cmd PRE EXEC /bin/vPre.tcl
#                                  custom-pre =allowGrp1 =allowGrp2
#         control the permissions for this command via glftpd.conf,
#         so make sure you bind only those groups, users, flags that
#         you want to use the command.
#
########################################################################


### Configuration ###

# (0=disable, 1=enable): give the user issuing SITE PRE the appropriate
# credits and stats for this pre.
set userfileUpdate 1

# Your datapath (relative to rootpath) as defined in glftpd's config
# file.
set ftpdataDir "/ftp-data"

# Some temp dir with chmod 0777 (relative to rootpath).
set tmpDir "/tmp2"

# Where can pre's be made from?  Case sensitive.  Can use '*' wildcard.
set BasePreDirs {
    /site/incoming/groups/*
}

# Sections that you want users to pre to.  Use lowercase only.
set PreTypes {
    tv
    ps2dvd
    mp3
}

# {dir matchFileType statSectionPosition ratio}
# Should have the same amount of lines as PreTypes.  Each line in
# PreTypes goes to these respective locations with the given ratios.
# So the second PreType goes to the second DestDir listed here.  The
# statSectionPosition is the stat_section as defined in your glftpd
# config.  Maybe you split the stats for iso and mp3?  Default (and also
# the first) section is 0.
set DestDirs {
    {/site/incoming/tv *.\[r0-9\]\[a0-9\]\[r0-9\] 0 3}
    {/site/incoming/console/ps2_dvd *.\[r0-9\]\[a0-9\]\[r0-9\] 0 3}
    {/site/mp3-today *.\[mM\]\[pP\]3 1 3}
}

#######################################################################







puts "pre script by vman (moopowah@yahoo.com); last revised $revision\n"


proc update_userfile { files size seconds section ratio } { global env ftpdataDir tmpDir

  #change $seconds from float to integer
  if { $seconds < 1.0 } {
        set seconds 1
  } else {
        set seconds [format "%d" $seconds]
  }
  set fout [open "$ftpdataDir/users/$env(USER).lock" w]
  puts $fout [pid]
  close $fout

  set fin [open "$ftpdataDir/users/$env(USER)" r]
  set fout [open "/tmp/$env(USER)_tempPreUserfile" w]

  while {![eof $fin]} {
        switch -exact -- [lindex [set currLine [gets $fin]] 0] {
                CREDITS {
                        # Add release's kbytes to old credit.
                        set currLine [lreplace $currLine [expr $section + 1] [expr $section + 1] [expr [lindex $currLine [expr $section + 1]] + ($ratio * $size)]]
                }
        }

# No more updating weektop stuffs
#               ALLUP -
#               WKUP -
#               DAYUP -
#               MONTHUP {
#                       set startIndex [expr ($section * 3) + 1]
#                       set endIndex [expr ($section * 3) + 1 + 2]
#                       #grab the old {files kbytes seconds}
#                       set oldStats [lrange $currLine $startIndex $endIndex]
#                       #update the stats
#                       set stats "[expr [lindex $oldStats 0] + $files] [expr [lindex $oldStats 1] + $size] [expr [lindex $oldStats 2] + $seconds]"
#                       #replace the indexes in the $currLine with the updated stats
#                       set currLine [lreplace $currLine $startIndex $startIndex [lindex $stats 0]]
#                       set currLine [lreplace $currLine [expr $startIndex + 1] [expr $startIndex + 1] [lindex $stats 1]]
#                       set currLine [lreplace $currLine [expr $startIndex + 2] [expr $startIndex + 2] [lindex $stats 2]]
#       }

        #write to new userfile
        puts $fout $currLine
  }

  close $fin
  close $fout
  #save old userfile, bring new userfile in its place
  #file rename -force -- "$ftpdataDir/users/$env(USER)" "$tmpDir/$env(USER).old"
  file rename -force -- "/tmp/$env(USER)_tempPreUserfile" "$ftpdataDir/users/$env(USER)"
  file delete -force -- "$ftpdataDir/users/$env(USER).lock"
  return 0
}


#verify user is in correct dir structure
set verified 0
foreach preDir $BasePreDirs {
  if { [string match $preDir [pwd]] } {
        set verified 1
        break
  }
}
if { $verified == 0 } {
  puts "Hi there.  I am Mr. PreScript and your bitchass has summoned me from"
  puts "the wrong directory!  Get your lame self to your private directory."
  exit 0
}

#insufficient arguments
if { [llength $argv] != 2 } {
  puts "Your syntax for SITE PRE is incorrect.  Observe the following syntax and examples:"
  puts "-> SITE PRE <myDir> <preType>"
  puts "Where preType is one of: $PreTypes"
  puts "For example:"
  puts "-> SITE PRE Quake_4-MYTH 0day"
  puts "-> SITE PRE 3D_Studio_MAX_v5.0-RISCISO app"
  exit 0
}

if { ![file isdirectory [lindex $argv 0]] } {
  puts "Yo babie, follow these steps:"
  puts "#1 take out \$20"
  puts "#2 buy yourself a good hammer from Home Depot"
  puts "#3 hit yourself on the head with your new hammer"
  puts "\"[pwd]/[lindex $argv 0]\" doesn't exist!"
  exit 0
}

set pre_location [lsearch $PreTypes [string tolower [lindex $argv 1]]]
if { $pre_location == -1 } {
  puts "\"[lindex $argv 1]\" is an invalid pre_type.  Please choose amongst: $PreTypes"
  exit 0
}

if {($userfileUpdate == 1) && ([file exists "$ftpdataDir/users/$env(USER).lock"])} {
        puts "Your userfile has a lock on it (it is being changed by some other process).  You may not pre yet!"
        exit 1
}

#get file names, pattern to search for
set release [lindex $argv 0]
set dest [lindex [lindex $DestDirs $pre_location] 0]
set filePattern [lindex [lindex $DestDirs $pre_location] 1]
cd $release
set AllFiles [glob -nocomplain -- *]

#gather release size, file amount
set PreFiles [glob -nocomplain -- $filePattern]
foreach aFile $AllFiles {
  if { [file isdirectory $aFile] } {
        append PreFiles " [glob -nocomplain -- $aFile/$filePattern]"
  }
}

#compute the values gathered above
set size 0      ;#in kilobytes
set files [llength $PreFiles]
set files_offset 0      ;#error fix of NULL files found
foreach aFile $PreFiles {
  if { $aFile == "" } {
        set files_offset [expr $files_offset - 1]
        continue
  }
  set size [expr ([file size $aFile] / 1024) + $size]
}
cd ..           ;#make sure to be back a dir!
set files_offset [expr $files_offset + [llength $PreFiles]]

#start reporting stuff here!
puts "Transferring [pwd]/$release -> $dest/$release..."
puts "Files: $files"
puts "Size: ${size}kB"


#copy the release and grab the elapsed time in seconds (double)
set tmp_file "$env(USER).pre.[clock seconds]"
#use `cp -R' or `mv'
if { [catch {exec -- time -p -o $tmpDir/$tmp_file mv -f [pwd]/$release $dest/} z] } {
  puts "error: moving/copying: $z"
  exit 1
}
set of [open $tmpDir/$tmp_file r]
set seconds [lindex [gets $of] 1]
catch { close $of }
file delete -force -- $tmpDir/$tmp_file

# log it all
set log_format [clock format [clock seconds] -format "%a %b %d %T %Y"]
set of [open $ftpdataDir/logs/glftpd.log a]

# style1:
#puts $of "$log_format PRE: \"$dest/$release\" \"$env(USER)\" \"$env(GROUP)\" $files $size"

# style2:
#puts $of "$log_format PRE: \"$dest/$release\" [string toupper [lindex $PreTypes $pre_location]] $env(USER) $env(GROUP) [expr $size / 1024] $files"

# style3:
puts $of "$log_format PRE: \"\[XXX\] New PRE by $env(USER)@$env(GROUP) - ($files files @ [expr $size / 1024]MB) --> $release\""


catch { close $of }

puts "Seconds: $seconds"
if { $userfileUpdate } {
        puts "Updating userfile..."
        update_userfile $files $size $seconds [lindex [lindex $DestDirs $pre_location] 2] [lindex [lindex $DestDirs $pre_location] 3]
}
puts "Cleaning up..."
catch { file delete -force -- [pwd]/$release }
puts "PRE SUCCESSFUL!"
puts ""
puts "Thank you for the pre.  Please exit to the left. :)"
puts "(milk refreshments at 50c a glass, straight ahead)"

#see ya!
exit 0

# EOF. :-)
